@@ -86,7 +86,7 @@ mod or_chip {
     use crate::circuit::*;
     use components_arena::ComponentId;
     use dep_obj::dep_type;
-    use dyn_context::{Context, ContextExt};
+    use dyn_context::{State, StateExt};
 
     dep_type! {
         #[derive(Debug)]
@@ -112,11 +112,11 @@ mod or_chip {
             result
         }
 
-        fn update(context: &mut dyn Context, chip: Chip, _old: &bool) {
-            let circuit: &Circuit= context.get();
+        fn update(state: &mut dyn State, chip: Chip, _old: &bool) {
+            let circuit: &Circuit = state.get();
             let in_1 = *chip.legs_ref(circuit).get(OrLegs::IN_1);
             let in_2 = *chip.legs_ref(circuit).get(OrLegs::IN_2);
-            chip.legs_mut(context).set_distinct(OrLegs::OUT, in_1 | in_2);
+            chip.legs_mut(state).set_distinct(OrLegs::OUT, in_1 | in_2);
         }
     }
 
@@ -127,7 +127,7 @@ mod not_chip {
     use crate::circuit::*;
     use components_arena::ComponentId;
     use dep_obj::dep_type;
-    use dyn_context::{Context, ContextExt};
+    use dyn_context::{State, StateExt};
 
     dep_type! {
         #[derive(Debug)]
@@ -151,10 +151,10 @@ mod not_chip {
             result
         }
 
-        fn update(context: &mut dyn Context, chip: Chip, _old: &bool) {
-            let circuit: &Circuit = context.get();
+        fn update(state: &mut dyn State, chip: Chip, _old: &bool) {
+            let circuit: &Circuit = state.get();
             let in_ = *chip.legs_ref(circuit).get(NotLegs::IN_);
-            chip.legs_mut(context).set_distinct(NotLegs::OUT, !in_);
+            chip.legs_mut(state).set_distinct(NotLegs::OUT, !in_);
         }
     }
 
@@ -162,7 +162,8 @@ mod not_chip {
 }
 
 use circuit::*;
-use dyn_context::{Context, ContextExt, free_lifetimes};
+use dep_obj::Dispatcher;
+use dyn_context::{State, StateExt};
 use not_chip::*;
 use or_chip::*;
 use std::any::{Any, TypeId};
@@ -174,27 +175,30 @@ struct TriggerChips {
     pub not_2: Chip,
 }
 
-free_lifetimes! {
-    struct TriggerContext {
-        circuit: 'circuit mut Circuit,
-        chips: 'chips ref TriggerChips,
-    }
+struct TriggerState {
+    dispatcher: Dispatcher,
+    circuit: Circuit,
+    chips: TriggerChips,
 }
 
-impl Context for TriggerContext {
+impl State for TriggerState {
     fn get_raw(&self, ty: TypeId) -> Option<&dyn Any> {
-        if ty == TypeId::of::<Circuit>() {
-            Some(self.circuit())
+        if ty == TypeId::of::<Dispatcher>() {
+            Some(&self.dispatcher)
+        } else if ty == TypeId::of::<Circuit>() {
+            Some(&self.circuit)
         } else if ty == TypeId::of::<TriggerChips>() {
-            Some(self.chips())
+            Some(&self.chips)
         } else {
             None
         }
     }
 
     fn get_mut_raw(&mut self, ty: TypeId) -> Option<&mut dyn Any> {
-        if ty == TypeId::of::<Circuit>() {
-            Some(self.circuit_mut())
+        if ty == TypeId::of::<Dispatcher>() {
+            Some(&mut self.dispatcher)
+        } else if ty == TypeId::of::<Circuit>() {
+            Some(&mut self.circuit)
         } else {
             None
         }
@@ -203,43 +207,57 @@ impl Context for TriggerContext {
 
 fn main() {
     let mut circuit_token = CircuitToken::new().unwrap();
-    let circuit = &mut Circuit::new(&mut circuit_token);
-    let not_1 = NotLegs::new(circuit, |chip| (1usize, chip));
-    let not_2 = NotLegs::new(circuit, |chip| (2usize, chip));
-    let or_1 = OrLegs::new(circuit, |chip| (1usize, chip));
-    let or_2 = OrLegs::new(circuit, |chip| (2usize, chip));
-    let on_not_out_changed = |context: &mut dyn Context, not: Chip, _old: &_| {
-        let circuit: &Circuit = context.get();
-        let chips: &TriggerChips = context.get();
+    let mut circuit = Circuit::new(&mut circuit_token);
+    let not_1 = NotLegs::new(&mut circuit, |chip| (1usize, chip));
+    let not_2 = NotLegs::new(&mut circuit, |chip| (2usize, chip));
+    let or_1 = OrLegs::new(&mut circuit, |chip| (1usize, chip));
+    let or_2 = OrLegs::new(&mut circuit, |chip| (2usize, chip));
+    let on_not_out_changed = |state: &mut dyn State, not: Chip, _old: &_| {
+        let circuit: &Circuit = state.get();
+        let chips: &TriggerChips = state.get();
         let or = if not.tag::<usize>(circuit) == 1 { chips.or_2 } else { chips.or_1 };
         let &out = not.legs_ref(circuit).get(NotLegs::OUT);
-        or.legs_mut(context).set_uncond(OrLegs::IN_2, out);
+        or.legs_mut(state).set_uncond(OrLegs::IN_2, out);
     };
-    let on_or_out_changed = |context: &mut dyn Context, or: Chip, _old: &_| {
-        let circuit: &Circuit = context.get();
-        let chips: &TriggerChips = context.get();
+    let on_or_out_changed = |state: &mut dyn State, or: Chip, _old: &_| {
+        let circuit: &Circuit = state.get();
+        let chips: &TriggerChips = state.get();
         let not = if or.tag::<usize>(circuit) == 1 { chips.not_1 } else { chips.not_2 };
         let &out = or.legs_ref(circuit).get(OrLegs::OUT);
-        not.legs_mut(context).set_uncond(NotLegs::IN_, out);
+        not.legs_mut(state).set_uncond(NotLegs::IN_, out);
     };
-    not_1.legs(circuit).on_changed(NotLegs::OUT, on_not_out_changed);
-    not_2.legs(circuit).on_changed(NotLegs::OUT, on_not_out_changed);
-    or_1.legs(circuit).on_changed(OrLegs::OUT, on_or_out_changed);
-    or_2.legs(circuit).on_changed(OrLegs::OUT, on_or_out_changed);
-    not_1.legs(circuit).on_changed(NotLegs::OUT, |context, not_1, _old| {
-        let circuit: &Circuit = context.get();
+    not_1.legs(&mut circuit).on_changed(NotLegs::OUT, on_not_out_changed);
+    not_2.legs(&mut circuit).on_changed(NotLegs::OUT, on_not_out_changed);
+    or_1.legs(&mut circuit).on_changed(OrLegs::OUT, on_or_out_changed);
+    or_2.legs(&mut circuit).on_changed(OrLegs::OUT, on_or_out_changed);
+    not_1.legs(&mut circuit).on_changed(NotLegs::OUT, |state, not_1, _old| {
+        let circuit: &Circuit = state.get();
         let &out = not_1.legs_ref(circuit).get(NotLegs::OUT);
         println!("{}", if out { "0 -> 1" } else { "1 -> 0" });
     });
-    let chips = &TriggerChips { or_1, or_2, not_1, not_2 };
-    TriggerContextBuilder { circuit, chips, }.build_and_then(|context| {
-        or_1.legs_mut(context).set_distinct(OrLegs::IN_1, true);
-        or_1.legs_mut(context).set_distinct(OrLegs::IN_1, false);
-        or_2.legs_mut(context).set_distinct(OrLegs::IN_1, true);
-        or_2.legs_mut(context).set_distinct(OrLegs::IN_1, false);
-        or_1.legs_mut(context).set_distinct(OrLegs::IN_1, true);
-        or_1.legs_mut(context).set_distinct(OrLegs::IN_1, false);
-        or_2.legs_mut(context).set_distinct(OrLegs::IN_1, true);
-        or_2.legs_mut(context).set_distinct(OrLegs::IN_1, false);
-    });
+    let chips = TriggerChips { or_1, or_2, not_1, not_2 };
+    let state = &mut TriggerState {
+        dispatcher: Dispatcher::new(),
+        circuit,
+        chips,
+    };
+    or_1.legs_mut(state).set_distinct(OrLegs::IN_1, true);
+    assert!(Dispatcher::dispatch(state));
+    or_1.legs_mut(state).set_distinct(OrLegs::IN_1, true);
+    assert!(!Dispatcher::dispatch(state));
+    or_1.legs_mut(state).set_distinct(OrLegs::IN_1, false);
+    assert!(Dispatcher::dispatch(state));
+    or_2.legs_mut(state).set_distinct(OrLegs::IN_1, true);
+    assert!(Dispatcher::dispatch(state));
+    or_2.legs_mut(state).set_distinct(OrLegs::IN_1, false);
+    assert!(Dispatcher::dispatch(state));
+    or_1.legs_mut(state).set_distinct(OrLegs::IN_1, true);
+    assert!(Dispatcher::dispatch(state));
+    or_1.legs_mut(state).set_distinct(OrLegs::IN_1, false);
+    assert!(Dispatcher::dispatch(state));
+    or_2.legs_mut(state).set_distinct(OrLegs::IN_1, true);
+    assert!(Dispatcher::dispatch(state));
+    or_2.legs_mut(state).set_distinct(OrLegs::IN_1, false);
+    assert!(Dispatcher::dispatch(state));
+    assert!(!Dispatcher::dispatch(state));
 }
