@@ -41,7 +41,7 @@ pub mod example {
     //! }
     //!
     //! macro_attr! {
-    //!     #[derive(Context!, Debug)]
+    //!     #[derive(State!, Debug)]
     //!     pub struct MyApp {
     //!         my_dep_types: Arena<MyDepTypePrivateData>,
     //!     }
@@ -67,7 +67,7 @@ pub mod example {
 
     use crate::{dep_obj, dep_type};
     use components_arena::{Arena, Component, ComponentId, Id};
-    use dyn_context::Context;
+    use dyn_context::State;
 
     dep_type! {
         #[derive(Debug)]
@@ -94,7 +94,7 @@ pub mod example {
         my_dep_types: Arena<MyDepTypePrivateData>,
     }
 
-    Context!(() pub struct MyApp { .. });
+    State!(() pub struct MyApp { .. });
 
     impl MyDepTypeId {
         pub fn new(app: &mut MyApp) -> MyDepTypeId {
@@ -128,7 +128,7 @@ pub use core::option::Option as std_option_Option;
 #[doc(hidden)]
 pub use core::stringify as std_stringify;
 #[doc(hidden)]
-pub use dyn_context::Context as dyn_context_Context;
+pub use dyn_context::State as dyn_context_State;
 #[doc(hidden)]
 pub use generics::concat as generics_concat;
 #[doc(hidden)]
@@ -146,8 +146,9 @@ use core::fmt::Debug;
 use core::mem::replace;
 use core::ops::Range;
 use dyn_clone::{DynClone, clone_trait_object};
-use dyn_context::{Context, ContextExt};
+use dyn_context::{State, StateExt};
 use educe::Educe;
+use macro_attr_2018::macro_attr;
 use phantom_type::PhantomType;
 
 /// A type should satisfy this trait to be a dependency property type
@@ -163,7 +164,7 @@ pub struct DepPropEntry<OwnerId: ComponentId, PropType: DepPropType> {
     style: Option<PropType>,
     local: Option<PropType>,
     #[educe(Debug(ignore))]
-    on_changed: Vec<fn(context: &mut dyn Context, id: OwnerId, old: &PropType)>,
+    on_changed: Vec<fn(state: &mut dyn State, id: OwnerId, old: &PropType)>,
 }
 
 impl<OwnerId: ComponentId, PropType: DepPropType> DepPropEntry<OwnerId, PropType> {
@@ -190,7 +191,7 @@ pub enum DepVecChange<ItemType: DepPropType> {
 pub struct DepVecEntry<OwnerId: ComponentId, ItemType: DepPropType> {
     items: Vec<ItemType>,
     #[educe(Debug(ignore))]
-    on_changed: Vec<fn(context: &mut dyn Context, id: OwnerId, change: &DepVecChange<ItemType>)>,
+    on_changed: Vec<fn(state: &mut dyn State, id: OwnerId, change: &DepVecChange<ItemType>)>,
 }
 
 impl<OwnerId: ComponentId, ItemType: DepPropType> DepVecEntry<OwnerId, ItemType> {
@@ -206,7 +207,7 @@ impl<OwnerId: ComponentId, ItemType: DepPropType> DepVecEntry<OwnerId, ItemType>
 #[educe(Debug)]
 pub struct DepEventEntry<OwnerId: ComponentId, ArgsType> {
     #[educe(Debug(ignore))]
-    on_raised: Vec<fn(context: &mut dyn Context, id: OwnerId, args: &mut ArgsType)>,
+    on_raised: Vec<fn(state: &mut dyn State, id: OwnerId, args: &mut ArgsType)>,
 }
 
 impl<OwnerId: ComponentId, ArgsType> DepEventEntry<OwnerId, ArgsType> {
@@ -226,9 +227,10 @@ impl<OwnerId: ComponentId, ArgsType> DepEventEntry<OwnerId, ArgsType> {
 /// # #![feature(const_ptr_offset_from)]
 /// # #![feature(const_raw_ptr_deref)]
 /// use components_arena::{Arena, Component, ComponentClassToken, ComponentId, Id};
-/// use dep_obj::{dep_obj, dep_type};
-/// use dyn_context::Context;
+/// use dep_obj::{Dispatcher, dep_obj, dep_type};
+/// use dyn_context::State;
 /// use macro_attr_2018::macro_attr;
+/// use std::any::{Any, TypeId};
 ///
 /// dep_type! {
 ///     #[derive(Debug)]
@@ -250,10 +252,30 @@ impl<OwnerId: ComponentId, ArgsType> DepEventEntry<OwnerId, ArgsType> {
 ///     pub struct MyDepTypeId(Id<MyDepTypePrivateData>);
 /// }
 ///
-/// macro_attr! {
-///     #[derive(Context!, Debug)]
-///     pub struct MyApp {
-///         my_dep_types: Arena<MyDepTypePrivateData>,
+/// pub struct MyApp {
+///     dispatcher: Dispatcher,
+///     my_dep_types: Arena<MyDepTypePrivateData>,
+/// }
+///
+/// impl State for MyApp {
+///     fn get_raw(&self, ty: TypeId) -> Option<&dyn Any> {
+///         if ty == TypeId::of::<Dispatcher>() {
+///             Some(&self.dispatcher)
+///         } else if ty == TypeId::of::<MyApp>() {
+///             Some(self)
+///         } else {
+///             None
+///         }
+///     }
+///
+///     fn get_mut_raw(&mut self, ty: TypeId) -> Option<&mut dyn Any> {
+///         if ty == TypeId::of::<Dispatcher>() {
+///             Some(&mut self.dispatcher)
+///         } else if ty == TypeId::of::<MyApp>() {
+///             Some(self)
+///         } else {
+///             None
+///         }
 ///     }
 /// }
 ///
@@ -278,6 +300,7 @@ impl<OwnerId: ComponentId, ArgsType> DepEventEntry<OwnerId, ArgsType> {
 /// fn main() {
 ///     let mut my_dep_types_token = ComponentClassToken::new().unwrap();
 ///     let mut app = MyApp {
+///         dispatcher: Dispatcher::new(),
 ///         my_dep_types: Arena::new(&mut my_dep_types_token),
 ///     };
 ///     let id = MyDepTypeId::new(&mut app);
@@ -402,7 +425,7 @@ trait AnySetter<Owner: DepType>: Debug + DynClone + Send + Sync {
         &self,
         owner: &mut Owner,
         unapply: bool
-    ) -> Option<Box<dyn for<'a> FnOnce(&'a mut dyn Context, Owner::Id)>>;
+    ) -> Option<Box<dyn for<'a> FnOnce(&'a mut dyn State, Owner::Id)>>;
 }
 
 clone_trait_object!(<Owner: DepType> AnySetter<Owner>);
@@ -414,7 +437,7 @@ impl<Owner: DepType, PropType: DepPropType> AnySetter<Owner> for Setter<Owner, P
         &self,
         owner: &mut Owner,
         unapply: bool
-    ) -> Option<Box<dyn for<'a> FnOnce(&'a mut dyn Context, Owner::Id)>> {
+    ) -> Option<Box<dyn for<'a> FnOnce(&'a mut dyn State, Owner::Id)>> {
         let entry_mut = self.prop.entry_mut(owner);
         let on_changed = if entry_mut.local.is_some() {
             None
@@ -425,9 +448,9 @@ impl<Owner: DepType, PropType: DepPropType> AnySetter<Owner> for Setter<Owner, P
         let old = replace(&mut entry_mut.style, value);
         on_changed.map(|on_changed| {
             let old = old.unwrap_or_else(|| entry_mut.default.clone());
-            Box::new(move |context: &'_ mut dyn Context, id| {
+            Box::new(move |state: &'_ mut dyn State, id| {
                 for on_changed in on_changed {
-                    on_changed(context, id, &old);
+                    on_changed(state, id, &old);
                 }
             }) as _
         })
@@ -490,9 +513,51 @@ impl<Owner: DepType> Style<Owner> {
     }
 }
 
+macro_attr! {
+    #[derive(State!)]
+    pub struct Dispatcher {
+        queue: Vec<Box<dyn FnOnce(&mut dyn State)>>,
+        empty_vec: Option<Vec<Box<dyn FnOnce(&mut dyn State)>>>,
+    }
+}
+
+impl Dispatcher {
+    pub fn new() -> Dispatcher {
+        Dispatcher { queue: Vec::new(), empty_vec: Some(Vec::new()) }
+    }
+
+    pub fn enqueue(&mut self, f: Box<dyn FnOnce(&mut dyn State)>) {
+        self.queue.push(f);
+    }
+
+    pub fn dispatch(state: &mut dyn State) -> bool {
+        let dispatcher: &mut Dispatcher = state.get_mut();
+        if let Some(empty_vec) = dispatcher.empty_vec.take() {
+            let mut queue = replace(&mut dispatcher.queue, empty_vec);
+            if queue.is_empty() {
+                dispatcher.empty_vec.replace(queue);
+                return false;
+            }
+            loop {
+                for item in queue.drain(..) {
+                    item(state);
+                }
+                let dispatcher: &mut Dispatcher = state.get_mut();
+                queue = replace(&mut dispatcher.queue, queue);
+                if queue.is_empty() {
+                    dispatcher.empty_vec.replace(queue);
+                    return true;
+                }
+            }
+        } else {
+            true
+        }
+    }
+}
+
 pub trait DepObjBuilderCore<OwnerId: ComponentId> {
-    fn context(&self) -> &dyn Context;
-    fn context_mut(&mut self) -> &mut dyn Context;
+    fn state(&self) -> &dyn State;
+    fn state_mut(&mut self) -> &mut dyn State;
     fn id(&self) -> OwnerId;
 }
 
@@ -543,7 +608,7 @@ impl<'a, Owner: DepType, Arena> DepObjRefMut<'a, Owner, Arena> {
         &mut self,
         prop: DepProp<Owner, PropType>,
         callback: fn(
-            context: &mut dyn Context,
+            state: &mut dyn State,
             id: Owner::Id,
             old: &PropType
         ),
@@ -557,7 +622,7 @@ impl<'a, Owner: DepType, Arena> DepObjRefMut<'a, Owner, Arena> {
         &mut self,
         event: DepEvent<Owner, ArgsType>,
         callback: fn(
-            context: &mut dyn Context,
+            state: &mut dyn State,
             id: Owner::Id,
             args: &mut ArgsType
         ),
@@ -571,7 +636,7 @@ impl<'a, Owner: DepType, Arena> DepObjRefMut<'a, Owner, Arena> {
         &mut self,
         vec: DepVec<Owner, ItemType>,
         callback: fn(
-            context: &mut dyn Context,
+            state: &mut dyn State,
             id: Owner::Id,
             change: &DepVecChange<ItemType>
         ),
@@ -584,50 +649,78 @@ impl<'a, Owner: DepType, Arena> DepObjRefMut<'a, Owner, Arena> {
 
 pub struct DepObjMut<'a, Owner: DepType, Arena> {
     id: Owner::Id,
-    context: &'a mut dyn Context,
+    state: &'a mut dyn State,
     get_obj_mut: for<'b> fn(arena: &'b mut Arena, id: Owner::Id) -> &'b mut Owner,
 }
 
 impl<'a, Owner: DepType, Arena: 'static> DepObjMut<'a, Owner, Arena> {
     pub fn new(
         id: Owner::Id,
-        context: &'a mut dyn Context,
+        state: &'a mut dyn State,
         get_obj_mut: for<'b> fn(arena: &'b mut Arena, id: Owner::Id) -> &'b mut Owner,
     ) -> Self {
-        DepObjMut { id, context, get_obj_mut }
+        DepObjMut { id, state, get_obj_mut }
     }
 
     pub fn set_uncond<PropType: DepPropType>(
         &mut self,
         prop: DepProp<Owner, PropType>,
         value: PropType,
-    ) -> PropType {
-        let arena: &mut Arena = self.context.get_mut();
+    ) where Owner::Id: 'static {
+        self.set_uncond_and_then(|_, _, _| { }, prop, value);
+    }
+
+    pub fn set_uncond_and_then<PropType: DepPropType>(
+        &mut self,
+        f: impl FnOnce(&mut dyn State, Owner::Id, PropType) + 'static,
+        prop: DepProp<Owner, PropType>,
+        value: PropType
+    ) where Owner::Id: 'static {
+        let arena: &mut Arena = self.state.get_mut();
         let obj = (self.get_obj_mut)(arena, self.id);
         let entry_mut = prop.entry_mut(obj);
         let old = entry_mut.local.replace(value).unwrap_or_else(||
             entry_mut.style.as_ref().unwrap_or_else(|| entry_mut.default).clone()
         );
-        for on_changed in entry_mut.on_changed.clone() {
-            on_changed(self.context, self.id, &old);
-        }
-        old
+        let on_changed = entry_mut.on_changed.clone();
+        let dispatcher: &mut Dispatcher = self.state.get_mut();
+        let id = self.id;
+        dispatcher.enqueue(Box::new(move |state| {
+            for on_changed in on_changed {
+                on_changed(state, id, &old);
+            }
+            f(state, id, old);
+        }));
     }
 
     pub fn unset_uncond<PropType: DepPropType>(
         &mut self,
         prop: DepProp<Owner, PropType>,
-    ) -> Option<PropType> {
-        let arena: &mut Arena = self.context.get_mut();
+    ) -> bool where Owner::Id: 'static {
+        self.unset_uncond_and_then(|_, _, _| { }, prop)
+    }
+
+    pub fn unset_uncond_and_then<PropType: DepPropType>(
+        &mut self,
+        f: impl FnOnce(&mut dyn State, Owner::Id, PropType) + 'static,
+        prop: DepProp<Owner, PropType>,
+    ) -> bool where Owner::Id: 'static {
+        let arena: &mut Arena = self.state.get_mut();
         let obj = (self.get_obj_mut)(arena, self.id);
         let entry_mut = prop.entry_mut(obj);
         if let Some(old) = entry_mut.local.take() {
-            for on_changed in entry_mut.on_changed.clone() {
-                on_changed(self.context, self.id, &old);
-            }
-            Some(old)
+            let on_changed = entry_mut.on_changed.clone();
+            let dispatcher: &mut Dispatcher = self.state.get_mut();
+            let id = self.id;
+            dispatcher.enqueue(Box::new(move |state| {
+                for on_changed in on_changed {
+                    on_changed(state, id, &old);
+                }
+                f(state, id, old);
+            }));
+            true
         } else {
-            None
+            false
         }
     }
 
@@ -635,61 +728,105 @@ impl<'a, Owner: DepType, Arena: 'static> DepObjMut<'a, Owner, Arena> {
         &mut self,
         prop: DepProp<Owner, PropType>,
         value: PropType,
-    ) -> Option<PropType> {
-        let arena: &mut Arena = self.context.get_mut();
+    ) -> bool where Owner::Id: 'static {
+        self.set_distinct_and_then(|_, _, _| { }, prop, value)
+    }
+
+    pub fn set_distinct_and_then<PropType: DepPropType + PartialEq>(
+        &mut self,
+        f: impl FnOnce(&mut dyn State, Owner::Id, PropType) + 'static,
+        prop: DepProp<Owner, PropType>,
+        value: PropType,
+    ) -> bool where Owner::Id: 'static {
+        let arena: &mut Arena = self.state.get_mut();
         let obj = (self.get_obj_mut)(arena, self.id);
         let entry_mut = prop.entry_mut(obj);
         let old = entry_mut.local.as_ref()
             .or_else(|| entry_mut.style.as_ref())
             .unwrap_or_else(|| entry_mut.default)
         ;
-        if &value == old { return None; }
+        if &value == old { return false; }
         let old = entry_mut.local.replace(value).unwrap_or_else(||
             entry_mut.style.as_ref().unwrap_or_else(|| entry_mut.default).clone()
         );
-        for on_changed in entry_mut.on_changed.clone() {
-            on_changed(self.context, self.id, &old);
-        }
-        Some(old)
+        let on_changed = entry_mut.on_changed.clone();
+        let dispatcher: &mut Dispatcher = self.state.get_mut();
+        let id = self.id;
+        dispatcher.enqueue(Box::new(move |state| {
+            for on_changed in on_changed {
+                on_changed(state, id, &old);
+            }
+            f(state, id, old);
+        }));
+        true
     }
 
     pub fn unset_distinct<PropType: DepPropType + PartialEq>(
         &mut self,
         prop: DepProp<Owner, PropType>,
-    ) -> Option<PropType> {
-        let arena: &mut Arena = self.context.get_mut();
+    ) -> bool where Owner::Id: 'static {
+        self.unset_distinct_and_then(|_, _, _| { }, prop)
+    }
+
+    pub fn unset_distinct_and_then<PropType: DepPropType + PartialEq>(
+        &mut self,
+        f: impl FnOnce(&mut dyn State, Owner::Id, PropType) + 'static,
+        prop: DepProp<Owner, PropType>,
+    ) -> bool where Owner::Id: 'static {
+        let arena: &mut Arena = self.state.get_mut();
         let obj = (self.get_obj_mut)(arena, self.id);
         let entry_mut = prop.entry_mut(obj);
         if let Some(old) = entry_mut.local.take() {
             let new = entry_mut.style.as_ref().unwrap_or_else(|| entry_mut.default);
-            if new == &old { return None; }
-            for on_changed in entry_mut.on_changed.clone() {
-                on_changed(self.context, self.id, &old);
-            }
-            Some(old)
+            if new == &old { return false; }
+            let on_changed = entry_mut.on_changed.clone();
+            let dispatcher: &mut Dispatcher = self.state.get_mut();
+            let id = self.id;
+            dispatcher.enqueue(Box::new(move |state| {
+                for on_changed in on_changed {
+                    on_changed(state, id, &old);
+                }
+                f(state, id, old);
+            }));
+            true
         } else {
-            None
+            false
         }
     }
 
-    pub fn raise<ArgsType>(
+    pub fn raise<ArgsType: 'static>(
         &mut self,
         event: DepEvent<Owner, ArgsType>,
-        args: &mut ArgsType,
-    ) {
-        let arena: &mut Arena = self.context.get_mut();
+        args: ArgsType,
+    ) where Owner::Id: 'static {
+        self.raise_and_then(|_, _, _| { }, event, args);
+    }
+
+    pub fn raise_and_then<ArgsType: 'static>(
+        &mut self,
+        f: impl FnOnce(&mut dyn State, Owner::Id, ArgsType) + 'static,
+        event: DepEvent<Owner, ArgsType>,
+        mut args: ArgsType,
+    ) where Owner::Id: 'static {
+        let arena: &mut Arena = self.state.get_mut();
         let obj = (self.get_obj_mut)(arena, self.id);
         let entry = event.entry(obj);
-        for on_raised in entry.on_raised.clone() {
-            on_raised(self.context, self.id, args);
-        }
+        let on_raised = entry.on_raised.clone();
+        let dispatcher: &mut Dispatcher = self.state.get_mut();
+        let id = self.id;
+        dispatcher.enqueue(Box::new(move |state| {
+            for on_raised in on_raised {
+                on_raised(state, id, &mut args);
+            }
+            f(state, id, args)
+        }));
     }
 
     pub fn apply_style(
         &mut self,
         style: Option<Style<Owner>>,
-    ) -> Option<Style<Owner>> {
-        let arena: &mut Arena = self.context.get_mut();
+    ) -> Option<Style<Owner>> where Owner::Id: 'static {
+        let arena: &mut Arena = self.state.get_mut();
         let obj = (self.get_obj_mut)(arena, self.id);
         let mut on_changed = Vec::new();
         let old = obj.style__().take();
@@ -715,43 +852,74 @@ impl<'a, Owner: DepType, Arena: 'static> DepObjMut<'a, Owner, Arena> {
             ;
         }
         *obj.style__() = style;
-        for on_changed in on_changed {
-            on_changed(self.context, self.id);
-        }
+        let dispatcher: &mut Dispatcher = self.state.get_mut();
+        let id = self.id;
+        dispatcher.enqueue(Box::new(move |state| {
+            for on_changed in on_changed {
+                on_changed(state, id);
+            }
+        }));
         old
     }
 
     pub fn clear<ItemType: DepPropType>(
         &mut self,
         vec: DepVec<Owner, ItemType>,
-    ) -> DepVecChange<ItemType> {
-        let arena: &mut Arena = self.context.get_mut();
+    ) where Owner::Id: 'static {
+        self.clear_and_then(|_, _, _| { }, vec);
+    }
+
+    pub fn clear_and_then<ItemType: DepPropType>(
+        &mut self,
+        f: impl FnOnce(&mut dyn State, Owner::Id, DepVecChange<ItemType>) + 'static,
+        vec: DepVec<Owner, ItemType>,
+    ) where Owner::Id: 'static {
+        let arena: &mut Arena = self.state.get_mut();
         let obj = (self.get_obj_mut)(arena, self.id);
         let entry_mut = vec.entry_mut(obj);
         let old_items = replace(&mut entry_mut.items, Vec::new());
         let change = DepVecChange::Reset(old_items);
-        for on_changed in entry_mut.on_changed.clone() {
-            on_changed(self.context, self.id, &change);
-        }
-        change
+        let on_changed = entry_mut.on_changed.clone();
+        let dispatcher: &mut Dispatcher = self.state.get_mut();
+        let id = self.id;
+        dispatcher.enqueue(Box::new(move |state| {
+            for on_changed in on_changed {
+                on_changed(state, id, &change);
+            }
+            f(state, id, change);
+        }));
     }
 
     pub fn push<ItemType: DepPropType>(
         &mut self,
         vec: DepVec<Owner, ItemType>,
         item: ItemType,
-    ) -> DepVecChange<ItemType> {
-        let arena: &mut Arena = self.context.get_mut();
+    ) where Owner::Id: 'static {
+        self.push_and_then(|_, _, _| { }, vec, item);
+    }
+
+    pub fn push_and_then<ItemType: DepPropType>(
+        &mut self,
+        f: impl FnOnce(&mut dyn State, Owner::Id, DepVecChange<ItemType>) + 'static,
+        vec: DepVec<Owner, ItemType>,
+        item: ItemType,
+    ) where Owner::Id: 'static {
+        let arena: &mut Arena = self.state.get_mut();
         let obj = (self.get_obj_mut)(arena, self.id);
         let entry_mut = vec.entry_mut(obj);
         entry_mut.items.push(item);
         let change = DepVecChange::Inserted(
             unsafe { entry_mut.items.len().unchecked_sub(1) } .. entry_mut.items.len()
         );
-        for on_changed in entry_mut.on_changed.clone() {
-            on_changed(self.context, self.id, &change);
-        }
-        change
+        let on_changed = entry_mut.on_changed.clone();
+        let dispatcher: &mut Dispatcher = self.state.get_mut();
+        let id = self.id;
+        dispatcher.enqueue(Box::new(move |state| {
+            for on_changed in on_changed {
+                on_changed(state, id, &change);
+            }
+            f(state, id, change);
+        }));
     }
 
     pub fn insert<ItemType: DepPropType>(
@@ -759,24 +927,48 @@ impl<'a, Owner: DepType, Arena: 'static> DepObjMut<'a, Owner, Arena> {
         vec: DepVec<Owner, ItemType>,
         index: usize,
         item: ItemType
-    ) -> DepVecChange<ItemType> {
-        let arena: &mut Arena = self.context.get_mut();
+    ) where Owner::Id: 'static {
+        self.insert_and_then(|_, _, _| { }, vec, index, item);
+    }
+
+    pub fn insert_and_then<ItemType: DepPropType>(
+        &mut self,
+        f: impl FnOnce(&mut dyn State, Owner::Id, DepVecChange<ItemType>) + 'static,
+        vec: DepVec<Owner, ItemType>,
+        index: usize,
+        item: ItemType
+    ) where Owner::Id: 'static {
+        let arena: &mut Arena = self.state.get_mut();
         let obj = (self.get_obj_mut)(arena, self.id);
         let entry_mut = vec.entry_mut(obj);
         entry_mut.items.insert(index, item);
         let change = DepVecChange::Inserted(index .. index + 1);
-        for on_changed in entry_mut.on_changed.clone() {
-            on_changed(self.context, self.id, &change);
-        }
-        change
+        let on_changed = entry_mut.on_changed.clone();
+        let dispatcher: &mut Dispatcher = self.state.get_mut();
+        let id = self.id;
+        dispatcher.enqueue(Box::new(move |state| {
+            for on_changed in on_changed {
+                on_changed(state, id, &change);
+            }
+            f(state, id, change);
+        }));
     }
 
     pub fn append<ItemType: DepPropType>(
         &mut self,
         vec: DepVec<Owner, ItemType>,
         other: &mut Vec<ItemType>,
-    ) -> DepVecChange<ItemType> {
-        let arena: &mut Arena = self.context.get_mut();
+    ) where Owner::Id: 'static {
+        self.append_and_then(|_, _, _| { }, vec, other);
+    }
+
+    pub fn append_and_then<ItemType: DepPropType>(
+        &mut self,
+        f: impl FnOnce(&mut dyn State, Owner::Id, DepVecChange<ItemType>) + 'static,
+        vec: DepVec<Owner, ItemType>,
+        other: &mut Vec<ItemType>,
+    ) where Owner::Id: 'static {
+        let arena: &mut Arena = self.state.get_mut();
         let obj = (self.get_obj_mut)(arena, self.id);
         let entry_mut = vec.entry_mut(obj);
         let appended = other.len();
@@ -784,10 +976,15 @@ impl<'a, Owner: DepType, Arena: 'static> DepObjMut<'a, Owner, Arena> {
         let change = DepVecChange::Inserted(
             unsafe { entry_mut.items.len().unchecked_sub(appended) } .. entry_mut.items.len()
         );
-        for on_changed in entry_mut.on_changed.clone() {
-            on_changed(self.context, self.id, &change);
-        }
-        change
+        let on_changed = entry_mut.on_changed.clone();
+        let dispatcher: &mut Dispatcher = self.state.get_mut();
+        let id = self.id;
+        dispatcher.enqueue(Box::new(move |state| {
+            for on_changed in on_changed {
+                on_changed(state, id, &change);
+            }
+            f(state, id, change);
+        }));
     }
 }
 
@@ -1201,8 +1398,8 @@ macro_rules! dep_type_impl_raw {
 
                     $vis fn $field(mut self, value: $field_ty) -> Self {
                         let id = <$BuilderCore as $crate::DepObjBuilderCore<$Id>>::id(&self.core);
-                        let context = <$BuilderCore as $crate::DepObjBuilderCore<$Id>>::context_mut(&mut self.core);
-                        id. [< $obj _mut >] (context).set_uncond($name:: [< $field:upper >] , value);
+                        let state = <$BuilderCore as $crate::DepObjBuilderCore<$Id>>::state_mut(&mut self.core);
+                        id. [< $obj _mut >] (state).set_uncond($name:: [< $field:upper >] , value);
                         self
                     }
                 ]
@@ -1450,9 +1647,9 @@ macro_rules! dep_obj {
             #[allow(dead_code)]
             $vis fn [< $name _mut >] <'arena_lifetime, DepObjType: $ty + $crate::DepType<Id=Self>>(
                 self,
-                context: &'arena_lifetime mut dyn $crate::dyn_context_Context,
+                state: &'arena_lifetime mut dyn $crate::dyn_context_State,
             ) -> $crate::DepObjMut<'arena_lifetime, DepObjType, $Arena> {
-                $crate::DepObjMut::new(self, context, Self:: [< $name _get_obj_mut_priv >] )
+                $crate::DepObjMut::new(self, state, Self:: [< $name _get_obj_mut_priv >] )
             }
 
             #[allow(dead_code)]
@@ -1495,9 +1692,9 @@ macro_rules! dep_obj {
             #[allow(dead_code)]
             $vis fn [< $name _mut >] <'arena_lifetime>(
                 self,
-                context: &'arena_lifetime mut dyn $crate::dyn_context_Context,
+                state: &'arena_lifetime mut dyn $crate::dyn_context_State,
             ) -> $crate::DepObjMut<'arena_lifetime, $ty, $Arena> {
-                $crate::DepObjMut::new(self, context, Self:: [< $name _get_obj_mut_priv >] )
+                $crate::DepObjMut::new(self, state, Self:: [< $name _get_obj_mut_priv >] )
             }
 
             #[allow(dead_code)]
@@ -1515,10 +1712,12 @@ macro_rules! dep_obj {
 mod test {
     use super::*;
     use alloc::boxed::Box;
+    use alloc::rc::Rc;
     use components_arena::{Arena, Id, ComponentId, Component, ComponentClassMutex};
-    use dyn_context::{Context, free_lifetimes, ContextRefMut};
+    use dyn_context::{State, free_lifetimes, StateRefMut};
     use educe::Educe;
     use macro_attr_2018::macro_attr;
+    use core::cell::Cell;
 
     macro_attr! {
         #[derive(Debug, Component!)]
@@ -1549,17 +1748,17 @@ mod test {
 
     struct TestIdBuilder<'a> {
         id: TestId,
-        arena: &'a mut TestArena
+        state: &'a mut dyn State
     }
 
     impl<'a> DepObjBuilderCore<TestId> for TestIdBuilder<'a> {
         fn id(&self) -> TestId { self.id }
-        fn context(&self) -> &dyn Context { self.arena }
-        fn context_mut(&mut self) -> &mut dyn Context { self.arena }
+        fn state(&self) -> &dyn State { self.state }
+        fn state_mut(&mut self) -> &mut dyn State { self.state }
     }
 
     macro_attr! {
-        #[derive(Context!)]
+        #[derive(State!)]
         struct TestArena(Arena<TestNode>);
     }
 
@@ -1581,80 +1780,97 @@ mod test {
     }
 
     free_lifetimes! {
-        struct TestContext {
+        struct TestState {
             changed: 'changed mut u16,
         }
     }
     
-    Context!(() struct TestContext { .. });
+    State!(() struct TestState { .. });
 
     #[test]
     fn create_test_obj_1() {
+        let mut dispatcher = Dispatcher::new();
         let mut arena = TestArena(Arena::new(&mut TEST_NODE.lock().unwrap()));
         let id = arena.0.insert(|id| (TestNode { obj1: None }, TestId(id)));
         TestObj1::new(&mut arena, id);
         let mut changed = 0;
-        TestContextBuilder { changed: &mut changed }.build_and_then(|context| {
-            context.merge_mut_and_then(|context| {
-                let arena = context.get_mut::<TestArena>();
-                let v = id.obj1_ref(arena).get(TestObj1::INT_VAL);
-                assert_eq!(v, &42);
-                id.obj1(arena).on_changed(TestObj1::INT_VAL, |context, _, _| {
-                    let test_context = context.get_mut::<TestContext>();
-                    *test_context.changed_mut() += 1;
-                });
-                let test_context = context.get::<TestContext>();
-                assert_eq!(test_context.changed(), &0);
-                id.obj1_mut(context).set_uncond(TestObj1::INT_VAL, 43);
-                let test_context = context.get::<TestContext>();
-                assert_eq!(test_context.changed(), &1);
-                let arena = context.get::<TestArena>();
-                assert_eq!(id.obj1_ref(arena).get(TestObj1::INT_VAL), &43);
+        TestStateBuilder { changed: &mut changed }.build_and_then(|state| {
+            state.merge_mut_and_then(|state| {
+                state.merge_mut_and_then(|state| {
+                    let arena = state.get_mut::<TestArena>();
+                    let v = id.obj1_ref(arena).get(TestObj1::INT_VAL);
+                    assert_eq!(v, &42);
+                    id.obj1(arena).on_changed(TestObj1::INT_VAL, |state, _, _| {
+                        let test_state = state.get_mut::<TestState>();
+                        *test_state.changed_mut() += 1;
+                    });
+                    let test_state = state.get::<TestState>();
+                    assert_eq!(test_state.changed(), &0);
+                    id.obj1_mut(state).set_uncond(TestObj1::INT_VAL, 43);
+                    let test_state = state.get::<TestState>();
+                    assert_eq!(test_state.changed(), &0);
+                    Dispatcher::dispatch(state);
+                    let test_state = state.get::<TestState>();
+                    assert_eq!(test_state.changed(), &1);
+                    let arena = state.get::<TestArena>();
+                    assert_eq!(id.obj1_ref(arena).get(TestObj1::INT_VAL), &43);
+                }, &mut dispatcher);
             }, &mut arena);
         });
     }
 
     #[test]
     fn test_obj_1_style() {
+        let mut dispatcher = Dispatcher::new();
         let mut arena = TestArena(Arena::new(&mut TEST_NODE.lock().unwrap()));
         let id = arena.0.insert(|id| (TestNode { obj1: None }, TestId(id)));
         TestObj1::new(&mut arena, id);
         assert_eq!(id.obj1_ref(&arena).get(TestObj1::INT_VAL), &42);
         let mut style = Style::new();
         style.insert(TestObj1::INT_VAL, 43);
-        id.obj1_mut(&mut arena).apply_style(Some(style.clone()));
+        (&mut arena).merge_mut_and_then(|s| id.obj1_mut(s).apply_style(Some(style.clone())), &mut dispatcher);
         assert_eq!(id.obj1_ref(&arena).get(TestObj1::INT_VAL), &43);
-        id.obj1_mut(&mut arena).set_uncond(TestObj1::INT_VAL, 44);
+        (&mut arena).merge_mut_and_then(|s| id.obj1_mut(s).set_uncond(TestObj1::INT_VAL, 44), &mut dispatcher);
         assert_eq!(id.obj1_ref(&arena).get(TestObj1::INT_VAL), &44);
         style.insert(TestObj1::INT_VAL, 45);
-        id.obj1_mut(&mut arena).apply_style(Some(style));
+        (&mut arena).merge_mut_and_then(|s| id.obj1_mut(s).apply_style(Some(style)), &mut dispatcher);
         assert_eq!(id.obj1_ref(&arena).get(TestObj1::INT_VAL), &44);
-        id.obj1_mut(&mut arena).unset_uncond(TestObj1::INT_VAL);
+        (&mut arena).merge_mut_and_then(|s| id.obj1_mut(s).unset_uncond(TestObj1::INT_VAL), &mut dispatcher);
         assert_eq!(id.obj1_ref(&arena).get(TestObj1::INT_VAL), &45);
     }
 
     #[test]
     fn test_obj_1_builder() {
+        let mut dispatcher = Dispatcher::new();
         let mut arena = TestArena(Arena::new(&mut TEST_NODE.lock().unwrap()));
         let id = arena.0.insert(|id| (TestNode { obj1: None }, TestId(id)));
         TestObj1::new(&mut arena, id);
-        let builder = TestObj1Builder::new_priv(TestIdBuilder { id, arena: &mut arena });
-        builder.int_val(1);
+        (&mut arena).merge_mut_and_then(|state| {
+            let builder = TestObj1Builder::new_priv(TestIdBuilder { id, state });
+            builder.int_val(1);
+        }, &mut dispatcher);
         assert_eq!(id.obj1_ref(&arena).get(TestObj1::INT_VAL), &1);
     }
 
     #[test]
     fn test_obj_1_coll() {
+        let mut dispatcher = Dispatcher::new();
         let mut arena = TestArena(Arena::new(&mut TEST_NODE.lock().unwrap()));
         let id = arena.0.insert(|id| (TestNode { obj1: None }, TestId(id)));
         TestObj1::new(&mut arena, id);
         assert!(id.obj1_ref(&arena).items(TestObj1::COLL).is_empty());
-        let change = id.obj1_mut(&mut arena).push(TestObj1::COLL, 7);
-        let change_match = match &change {
-            DepVecChange::Inserted(x) => *x == (0 .. 1),
-            _ => false,
-        };
-        assert!(change_match, "{:?}", change);
+        let change_match = Rc::new(Cell::new(false));
+        let change_match_ref = change_match.clone();
+        (&mut arena).merge_mut_and_then(|state| {
+            id.obj1_mut(state).push_and_then(move |_, _, change| {
+                change_match_ref.set(match &change {
+                    DepVecChange::Inserted(x) => *x == (0 .. 1),
+                    _ => false,
+                });
+            }, TestObj1::COLL, 7);
+            Dispatcher::dispatch(state);
+        }, &mut dispatcher);
+        assert!(change_match.get());
         assert_eq!(id.obj1_ref(&arena).items(TestObj1::COLL)[0], 7);
     }
 }
